/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

#ifndef CONFIG_H
#define CONFIG_H

#include <stdbool.h>

//compile time configuration
#define IMPLEMENTOR_NAME "fnyback"
#define PROGRAM_NAME "http-server"
#define SERVER_LOG_FILENAME "http-server-log.txt"
#define DEFAULT_ROOT "files"
#define BUFFERSIZE 8192
#define DISCOVERY_HOSTNAME "nwprog1.netlab.hut.fi"
#define DISCOVERY_PORT "3000"

//general macros
#define MIN(x,y) ((x) < (y) ? (x) : (y))
#define MAX(x,y) ((x) > (y) ? (x) : (y))
#define ARRAY_LENGTH(a) (sizeof(a) / sizeof(a[0]))

//type definitions
typedef unsigned long long file_size_t;

//struct for converting an internal status code to a HTTP status code,
//a reason phrase and body message
struct HTTP_STATUS {
	int status_code;
	const char *reason_phrase;
	const char *body_message;
};

//internal status codes
enum STATUS {
	STATUS_OK, //200
	STATUS_CREATED, //201
	STATUS_BAD_REQUEST, //400
	STATUS_SYNTAX_ERROR_IN_REQUEST_LINE, //400
	STATUS_SYNTAX_ERROR_IN_REQUEST_PATH, //400
	STATUS_SYNTAX_ERROR_IN_HEADER_LINE, //400
	STATUS_DIRECTORY, //403
	STATUS_NOT_FOUND, //404
	STATUS_METHOD_NOT_ALLOWED, //405
	STATUS_LENGTH_REQUIRED, //411
	STATUS_UNSUPPORTED_MEDIA_TYPE, //415
	STATUS_INTERNAL_SERVER_ERROR, //500
	STATUS_NOT_IMPLEMENTED, //501
};
//a shorter alias
#define STATUS_ERROR STATUS_INTERNAL_SERVER_ERROR

//kept here to prevent enum STATUS and the lookup table to get out of sync
#define HTTP_STATUS_DATA { \
	{200, "OK", "OK"}, \
	{201, "Created", "Created"}, \
	{400, "Bad Request", "Bad Request"}, \
	{400, "Bad Request", "Bad Request: Syntax error in request line."}, \
	{400, "Bad Request", "Bad Request: Syntax error in request path."}, \
	{400, "Bad Request", "Bad Request: Syntax error in header line."}, \
	{403, "Access Denied", "Access Denied: Request for Directory."}, \
	{404, "Not Found", "Not Found"}, \
	{405, "Method Not Allowed", "Method Not Allowed"}, \
	{411, "Length Required", "Length Required. The Content-Length header line was missing or malformed."}, \
	{415, "Unsupported Media Type", "Unsupported Media Type"}, \
	{500, "Internal Server Error", "Internal Server Error"}, \
	{501, "Not Implemented", "Not Implemented"}, \
	}


//run time configuration
struct CONFIG {
	int backlog_length;
	unsigned int port;
	const char *root;
	bool ipv4_only;
	bool ipv6_only;
	bool daemonize;
	bool discovery;
	int max_threads;
	int backlog;
	
	int server_sock_fd;
};

#endif //CONFIG_H
