/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

//standard C headers
#include <stdio.h>
#include <string.h>

//unix headers
#include <unistd.h>

//network headers
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>

//local headers
#include "connection.h"
#include "log.h"

/**
 * Connect to the specified hostname and port.
 *
 * @param hostname The hostname to connect to.
 * @param port The port to connect to.
 * @return Returns a stream (FILE *) handle on success or NULL on failure.
 */
FILE *create_tcp_connection(const char *hostname, const char *port)
{
	struct addrinfo *result = NULL, *r, hints;
	int socket_fd;
	FILE *socket_stream = NULL;
	char addr_buffer[100];

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;

	if(getaddrinfo(hostname, port, &hints, &result) != 0) {
		log_printf(LL_ERRNO, "getaddrinfo() failed.");
		return NULL;
	}
	
	if(result == NULL) {
		log_printf(LL_ERRNO, "result from get addrinfo is NULL.");
		return NULL;
	}
	
	//try all provided addresses
	for(r = result; r != NULL; r = r->ai_next) {
		
		//convert IP (v4 or v6) address to string
		getnameinfo(r->ai_addr, r->ai_addrlen,
			addr_buffer, sizeof(addr_buffer),
			NULL, 0, NI_NUMERICHOST);
		log_printf(LL_INFO, "Connecting to %s.", addr_buffer);

		socket_fd = socket(r->ai_family, r->ai_socktype, 0);
		if(socket_fd < 0) {
			log_printf(LL_ERRNO, "socket() failed.");
			continue;
		}
	
		if(connect(socket_fd, r->ai_addr, r->ai_addrlen) != 0) {
			log_printf(LL_ERRNO, "connect() failed.");
			close(socket_fd);
			continue;
		}
		
		break;
	}
	
	if(result != NULL) {
		freeaddrinfo(result);
	}
	
	if(socket_fd >= 0)
		socket_stream = fdopen(socket_fd, "r+b");
	
	return socket_stream;	
}

/**
 * Get the (canonical) hostname of the server itself.
 *
 * @param buffer Buffer to place the hostname in.
 * @param buffer_length Length of the buffer.
 * @return Returns true on success, false on failure.
 */
bool get_own_hostname(char *buffer, size_t buffer_length)
{
	bool ok = false;
	char hostname_buffer[1024];
	struct addrinfo *result = NULL, hints;

	do {

		//this is not the complete (FQDN) hostname yet
		if(gethostname(hostname_buffer, sizeof(hostname_buffer)) != 0) {
			log_printf(LL_ERRNO, "gethostname failed.");
			break;
		}

		//ask for the canonical hostname
		memset(&hints, 0, sizeof(hints));
		hints.ai_family = AF_UNSPEC;
		hints.ai_socktype = SOCK_STREAM;
		hints.ai_flags = AI_CANONNAME;
		if(getaddrinfo(hostname_buffer, NULL, &hints, &result) != 0) {
			log_printf(LL_ERRNO, "getaddrinfo failed.");
			break;
		}
		
		//ai_canonname of the first result is the official name of the host
		//check that it is valid
		if(result == NULL || result->ai_canonname == NULL) {
			log_printf(LL_ERROR, "Result from getaddrinfo does not contain a canonical name.");
			break;
		}
		
		strncpy(buffer, result->ai_canonname, buffer_length);
	
		ok = true;
	} while(false);
	
	if(result != NULL)
		freeaddrinfo(result);
	
	return ok;
}
