/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

//standard C headers
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//unix headers
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

//local headers
#include "directory.h"
#include "log.h"

/**
 * Read directory entries and write them to a preallocated buffer.
 *
 * @param local_path The local path of the directory.
 * @param reply_header The header structure for the reply.
 * @param buffer The preallocated buffer to write the directory listing to.
 * @param buffer_length Length of the buffer.
 * @return Returns true on success or false on failure.
 */
bool list_directory(const char *local_path, struct HEADER *reply_header, 
	char *buffer, size_t buffer_length)
{
	DIR *dir;
	bool ok = false;
	struct dirent de, *de_ptr;
	struct stat file_stat;
	size_t len;
	int l;
	char *local_filename = NULL;
	size_t local_path_len, local_filename_len, de_filename_len;
	
	if(local_path == NULL || reply_header == NULL || buffer == NULL)
		return false;
	
	local_path_len = strlen(local_path);
	local_filename_len = local_path_len + 1 + sizeof(de.d_name) + 1;
	//allocate a buffer for the local_filename
	local_filename = (char *)malloc(local_filename_len);
	if(local_filename == NULL)
		return false;
	
	//prepare the local filename buffer by initializing it to
	//local_path/
	strncpy(local_filename, local_path, local_filename_len + 1);
	local_filename[local_path_len] = '/';
	
	do {
		//open directory
		dir = opendir(local_path);
		if(dir == NULL) {
			log_printf(LL_ERRNO, "Error opening directory '%s'.", local_path);
			break;
		}
		
		//add one directory entry at a time to the buffer
		for(len = 0; len < buffer_length; ) {
		
			if(readdir_r(dir, &de, &de_ptr) != 0)
				break;
			if(de_ptr == NULL)
				break;
	
			//omit hidden files
			if(de.d_name[0] == '.')
				continue;

			//create the local filename
			//local_filename = local_path + / + directory entry name
			de_filename_len = strlen(de.d_name);
			strncpy(local_filename + local_path_len + 1,
				de.d_name,
				de_filename_len + 1);
			//log_printf(LL_DEBUG, "local_filename='%s'.", local_filename);
		
			//stat the file to get the type and sizes
			if(stat(local_filename, &file_stat) != 0) {
				log_printf(LL_ERRNO, "stat failed for file '%s'.", local_filename);
				break;
			}

			if(S_ISDIR(file_stat.st_mode)) {
				l = snprintf(buffer + len, buffer_length - len,
						"%s (dir)\n", de.d_name);
			} else {
				l = snprintf(buffer + len, buffer_length - len,
					"%s %llu B\n", 
					de.d_name, (unsigned long long)file_stat.st_size);
			}

			if(l < 0)
				break;
			len += l;
		}
	
		//Change the content length in the reply header. This is the length of
		//the formatted directory listing, not the size of the dir on disk.
		reply_header->content_length = len;
		
		ok = true;
	} while(false);

	//clean up
	if(dir != NULL)
		closedir(dir);
	if(local_filename != NULL)
		free(local_filename);

	return ok;
}
