/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

//standard C headers
#include <stdio.h>
#include <string.h>

//network headers
#include <netinet/in.h>

//local headers
#include "config.h"
#include "connection.h"
#include "discovery.h"
#include "parse.h"
#include "log.h"

enum DISCOVERY_TYPE {DISCOVERY_JOIN, DISCOVERY_LEAVE};

/**
 * Send a discovery request, either of join type (announce the server to the
 * discovery service) or of leave type (remove the server from the discovery
 * service).
 *
 * @param type The type of the request: either DISCOVERY_JOIN or 
 *   DISCOVERY_LEAVE.
 * @param config Configuration data to use. The port number if got from here.
 * @return Returns true on success, false on failure.
 */
bool send_discovery_request(enum DISCOVERY_TYPE type, struct CONFIG *config)
{
	bool ok = false;
	struct HEADER *reply_header = NULL;
	FILE *sock = NULL;
	int sock_fd;
	//the discovery service doesn't seem to like empty files
	//with Content-Length: 0, so send a \n
	char buffer[1024] = "\n", *b;
	size_t len = 0;

	if((type != DISCOVERY_JOIN && type != DISCOVERY_LEAVE) 
		|| config == NULL)
		return false;

	do {
		sock = create_tcp_connection(DISCOVERY_HOSTNAME, DISCOVERY_PORT);
		if(sock == NULL)
			break;
		
		if(type == DISCOVERY_JOIN) {
			sock_fd = fileno(sock);
			if(sock_fd == -1)
				break;
		
			//get the hostname of this host
			if(!get_own_hostname(buffer, sizeof(buffer)))
				break;
			
			//append port
			len = strlen(buffer);
			b = buffer + len;
			snprintf(b, sizeof(buffer) - len,
				":%u\n", config->port);
			log_printf(LL_INFO, "Joining discovery service with address: %s",
				buffer);
		}
		len = strlen(buffer);
	
		//send the header and the body
		if(fprintf(sock,
			"PUT /servers-%s.txt HTTP/1.1\r\n"
			"Host: %s\r\n"
			"Content-Type: text/plain\r\n"
			"Content-Length: %zu\r\n"
			"Iam: %s\r\n"
			"Connection: close\r\n"
			"\r\n"
			"%s",
			IMPLEMENTOR_NAME,
			DISCOVERY_HOSTNAME,
			len,
			IMPLEMENTOR_NAME,
			buffer) < 0)
			break;
		
		fflush(sock);
		
		reply_header = parse_header(sock, HEADER_TYPE_REPLY, NULL);
		if(reply_header == NULL) {
			log_printf(LL_ERROR, "Error parsing header.");
			break;
		}
		
		log_printf(LL_INFO, "Discovery service reply status: '%s'", 
			reply_header->status_line);;
		if(reply_header->status_code[0] != '2') {
			break;
		}
		
		ok = true;
	} while(false);

	free_header(reply_header);
	if(sock != NULL)
		fclose(sock);

	return ok;
}

/**
 * Send a discovery request of join type (announce the server to the
 * discovery service).
 *
 * @param config Configuration data to use. The port number if got from here.
 * @return Returns true on success, false on failure.
 */
bool discovery_service_join(struct CONFIG *config)
{
	return send_discovery_request(DISCOVERY_JOIN, config);
}

/**
 * Send a discovery request of leave type (remove the server from the discovery
 * service).
 *
 * @param config Configuration data to use. The port number if got from here.
 * @return Returns true on success, false on failure.
 */
bool discovery_service_leave(struct CONFIG *config)
{
	return send_discovery_request(DISCOVERY_LEAVE, config);
}
