/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

#ifndef PARSE_H
#define PARSE_H

#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <time.h>

#include "config.h"

#define WHITESPACE " \t"
#define COLON_WHITESPACE " \t:"
#define DIGITS "0123456789"

enum HEADER_TYPE {
	HEADER_TYPE_REQUEST,
	HEADER_TYPE_REPLY
};

enum METHOD {
	METHOD_GET,
	METHOD_PUT
};

struct URL {
	const char *scheme;
	const char *hostname;
	const char *port;
	const char *path;
	const char *query;
	const char *filename;
};

struct HEADER {
	enum HEADER_TYPE type;

	//request
	const char *request_line;
	enum METHOD method;
	char *request_path;
	
	//request & reply
	unsigned int http_version_major;
	unsigned int http_version_minor;

	//reply
	const char *status_line;
	const char *status_code;
	bool content_length_valid;
	file_size_t content_length;
	const char *content_type;
	const char *iam;
	//sending reply
	enum STATUS status;
	time_t last_modified;	
};

struct HEADER *alloc_and_init_header(enum HEADER_TYPE type);
struct HEADER *parse_header(FILE *socket_stream, enum HEADER_TYPE type,
	enum STATUS *status);
bool free_header(struct HEADER *header);

bool decode_request_path(char *request_path);
bool validate_request_path(const char *request_path);

#endif //PARSE_H
