/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

//standard C headers
#include <stdlib.h>
#include <string.h>

//local headers
#include "string.h"
#include "pstring.h"

/**
 * Initialize the PSTRING with the substring starting at address s and having 
 * the specified length.
 *
 * @param ps The PSTRING to initialize.
 * @param s The address of the C substring data.
 * @param length  Length of the substring.
 * @return Returns true on success, false on failure.
 */
bool pstring_init(struct PSTRING *ps, const char *s, size_t length)
{
	if(ps == NULL)
		return false;
	
	ps->ptr = s;
	ps->length = length;
	
	return true;
}

/**
 * Clears the PSTRING struct effectively makeing it empty. Does not free the
 * memory used by the PSTRING.
 *
 * @param ps The PSTRING to clear.
 * @return Returns true on success, false on failure.
 */
bool pstring_clear(struct PSTRING *ps)
{
	if(ps == NULL)
		return false;
	
	return pstring_init(ps, NULL, 0);
}

/**
 * Convert the PSTRING to a C string.
 *
 * @param ps The PSTRING to convert.
 * @return Returns the converted C string.
 **/
const char *pstring_to_cstring(struct PSTRING *ps)
{
	if(ps == NULL)
		return NULL;
		
	if(ps->length == 0)
		return NULL;

	return strndup(ps->ptr, ps->length);
}

/**
 * Compare the PSTRING to a C string case sensitively.
 * Changes: The comparison also takes into account the length of the strings,
 * not only the matching characters. This was a bug in the first implementation
 * for the client.
 *
 * @param ps PSTRING to compare.
 * @param s C string to compare.
 * @return Returns true if the strings are equal, false otherwise.
 */
bool pstring_equals(struct PSTRING *ps, const char *s)
{
	return strlen(s) == ps->length && strncmp(ps->ptr, s, ps->length) == 0;
}

/**
 * Compare the PSTRING to a C string case insensitively.
 * Changes: The comparison also takes into account the length of the strings,
 * not only the matching characters. This was a bug in the first implementation
 * for the client.
 *
 * @param ps PSTRING to compare.
 * @param s C string to compare.
 * @return Returns true if the strings are equal, false otherwise.
 */
bool pstring_iequals(struct PSTRING *ps, const char *s)
{
	return strlen(s) == ps->length && strnicmp(ps->ptr, s, ps->length) == 0;
}
