/*
//
// S-38.3610 Network Programming (Spring 2013)
// Assignment 2: HTTP Server
// Filip Nybck (79808K)
//
*/

#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>

#include "config.h"
#include "log.h"
//#include "parse.h"
#include "transfer.h"

/**
 * Transfer binary data from one stream (FILE *) to another one buffer at a 
 * time.
 *
 * @param content_length Number of bytes to transfer.
 * @param from The source stream.
 * @param to The destination stream.
 */
bool transfer_binary_data(file_size_t content_length, FILE *from, FILE *to)
{
	file_size_t offset, chunk_length;
	char buffer[BUFFERSIZE];
	size_t rlength, wlength;
	
	if(from == NULL || to == NULL)
		return false;
	
	for(offset = 0; offset < content_length; offset += chunk_length) {
				
		chunk_length = MIN(content_length - offset, sizeof(buffer));
		rlength = fread(buffer, 1, chunk_length, from);
		if(rlength < 0) {
			log_printf(LL_ERRNO, "rlength < 0.");
			return false;
		} else if(rlength == 0) {
			log_printf(LL_ERRNO, "fread returned eof.");
			return false;
		} else if(rlength != chunk_length) {
			log_printf(LL_ERROR, "rlength != chunk_length.");
			return false;
		}
				
		wlength = fwrite(buffer, 1, chunk_length, to);
		if(wlength <= 0) {
			log_printf(LL_ERRNO, "wlength <= 0.");
			return false;
		} else if(wlength != chunk_length) {
			log_printf(LL_ERROR, "wlength != chunk_length.");
			return false;
		}
	}
	
	return true;
}

/**
 * Transfer binary data from a socket stream to a file stream one buffer at a 
 * time.
 *
 * @param content_length Number of bytes to transfer.
 * @param socket_stream The source stream.
 * @param local_file The destination stream.
 */
bool receive_binary_data(file_size_t content_length, FILE *socket_stream, 
	FILE *local_file)
{
	return transfer_binary_data(content_length, socket_stream, local_file);
}

/**
 * Transfer binary data from a file stream to a socket stream one buffer at a 
 * time.
 *
 * @param content_length Number of bytes to transfer.
 * @param socket_stream The destination stream.
 * @param local_file The source stream.
 */
bool send_binary_data(file_size_t content_length, FILE *socket_stream, 
	FILE *local_file)
{
	return transfer_binary_data(content_length, local_file, socket_stream);
}

/**
 * Transfer binary data from a buffer to a socket stream. A wrapper around 
 * fwrite.
 *
 * @param content_length Number of bytes to transfer.
 * @param socket_stream The destination stream.
 * @param buffer The source buffer.
 */
bool send_binary_mem_data(file_size_t content_length, FILE *socket_stream, 
	void *buffer)
{
	return fwrite(buffer, 1, content_length, socket_stream) == content_length;
}
